#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de proyecto. Este controlador es utilizado desde las secciones:
# * Sección administracion: se utiliza para crear proyectos
# * Sección proyectos: se utiliza para seleccionar el proyecto y cargar lo en la sessión

class ProyectoController < ApplicationController
  # en administracion y proyectos: se redirecciona por defecto a ordenado
  def index
    filtrado_ordenado_iniciales
    redirect_to :action => "listado" if params[:seccion] == "administracion"
    redirect_to :action => "listado_usuario" if params[:seccion] == "proyectos"
  end


  def filtrado_condiciones
    condiciones = "convenio_id IS NULL AND convenio_accion IS NULL" unless params[:menu] == "convenio"
    condiciones = "convenio_id IS NULL AND convenio_accion IS NOT NULL" if params[:menu] == "convenio"
    condiciones = "convenio_id IS NULL" if params[:seccion] == "proyectos"
    condiciones += " AND pais_principal_id = " + session[:proyecto_filtro_pais] unless session[:proyecto_filtro_pais] == "todos"
    condiciones += " AND agente_id = " + session[:proyecto_filtro_financiador] unless session[:proyecto_filtro_financiador] == "todos"
    condiciones += " AND fecha_convocatoria BETWEEN '#{session[:proyecto_filtro_ano]}-1-1' AND '#{(session[:proyecto_filtro_ano].to_i + 1)}-1-1' "  unless session[:proyecto_filtro_ano] == "todos"
    condiciones += " AND definicion_estado.id = " + session[:proyecto_filtro_estado] unless session[:proyecto_filtro_estado] == "todos" or session[:proyecto_filtro_estado] == "activos"
    condiciones += " AND definicion_estado.cerrado = 0" if session[:proyecto_filtro_estado] == "activos"
    @condiciones = condiciones
    paises = [["Todos", "todos"]] + Pais.all(:order => "nombre").collect {|p| [p.nombre, p.id.to_s]}
    anos = [["Todos", "todos"]] +  Array.new(100) {|x| [ x + 1980 , (x + 1980).to_s ]}
    agentes =  [["Todos", "todos"]] + Agente.all(:order => "nombre", :conditions => {:financiador => true }).collect {|p| [p.nombre, p.id.to_s]}
    estado =  [["Todos", "todos"]] + [["Estados abiertos","activos"]] + DefinicionEstado.all(:order => "nombre").collect {|p| [p.nombre, p.id.to_s]}
     @opciones_filtrado =  [{:rotulo => "Seleccione pais:", :nombre=> "pais", :opciones =>  paises}, 
          {:rotulo => "Seleccione Año:", :nombre=> "ano", :opciones => anos }, 
  {:rotulo => "Seleccione financiador:", :nombre=> "financiador", :opciones => agentes },
{:rotulo => "Seleccione estado:", :nombre=> "estado", :opciones => estado } ] 
    @accion_filtrado = {:action => :filtrado }
    
  end

  # en administracion: lista
  def listado
    filtrado_condiciones
    @proyectos = @paginado =  Proyecto.paginate( :all, :conditions => @condiciones, :order => (session[:proyecto_orden] + " " + session[:proyecto_asc_desc] ) , :include => [:pais_principal, :agente, :definicion_estado], :page => params[:page], :per_page => (session[:por_pagina]) )
  end

  def filtrado_ordenado_iniciales
    session[:proyecto_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"   
    session[:proyecto_orden] = params[:orden] ? params[:orden] : "proyecto.nombre"
    session[:proyecto_filtro_estado] = "todos"
    session[:proyecto_filtro_pais] = "todos"
    session[:proyecto_filtro_financiador] = "todos"
    session[:proyecto_filtro_ano] = "todos"
  end

  # en administracion y proyectos: establece los parametros de ordenación
  def ordenado
    session[:proyecto_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"   
    session[:proyecto_orden] = params[:orden] ? params[:orden] : "nombre"
    session[:proyecto_orden] = "pais.nombre" if params[:orden] == "pais_principal.nombre"
    redirect_to :action => "listado" if params[:seccion] == "administracion"
    redirect_to :action => "listado_usuario" if params[:seccion] == "proyectos"
  end

  def filtrado
    session[:proyecto_filtro_estado] = params[:filtro][:estado]
    session[:proyecto_filtro_pais] = params[:filtro][:pais]
    session[:proyecto_filtro_financiador] = params[:filtro][:financiador]
    session[:proyecto_filtro_ano] = params[:filtro][:ano]
    redirect_to :action => "listado" if params[:seccion] == "administracion"
    redirect_to :action => "listado_usuario" if params[:seccion] == "proyectos"
  end
  
  # en administracion: prepara el formulario de edición o creación
  def editar_nuevo
    @proyecto = params[:id] ? Proyecto.find(params[:id]) : nil 
    @paises = Pais.all(:order => "nombre").collect {|p| [p.nombre, p.id]}
    @financiadores = Agente.find_all_by_financiador(true, :order => "nombre").collect {|a| [a.nombre, a.id]}
    @monedas = Moneda.all(:order => "nombre").collect {|a| [a.nombre, a.id]}
    condicion_libros = {:tipo => "banco"} 
    condicion_libros[:moneda_id] = @proyecto.moneda_id if @proyecto
    @libros = Libro.all(:order => "nombre", :conditions => condicion_libros).collect {|a| [a.nombre, a.id]}
    @convenio_id = params[:convenio_id] if params[:convenio_id]
    render :partial => "formulario"
  end

  def libro_cambio_moneda
    libros = Libro.all(:order => "nombre", :conditions => {:tipo => "banco", :moneda_id => params[:moneda_id]}).collect {|a| [a.nombre, a.id]}
    render :update do |page|
      page.replace "selector_libro_financiador", :partial => "selector_libro_financiador", :locals => { :libros => libros }
    end
  end

  # en administracion: modifica o crea
  def modificar_crear
    @proyecto = params[:id] ?  Proyecto.find(params[:id]) : Proyecto.new
    params[:proyecto][:pais_principal_id] = "" if params[:selector].nil? || params[:selector][:multipais] == "1"
    @proyecto.update_attributes params[:proyecto]
    if params[:convenio_id]
      convenio = Proyecto.find_by_id params[:convenio_id]
      @proyectos = convenio.pacs
      render :update do |page|
        page.replace_html params[:update], :partial => "listado_proyectos_convenio"
        page.visual_effect :highlight, params[:update] , :duration => 6
        page.replace 'formulario', :inline => '<%= mensaje_error(@proyecto) %><br>'
        page.call("Modalbox.resizeToContent")
      end
    else
      flash[:error] = @proyecto
      redirect_to :action => "listado"
    end
  end

  # en administracion: elimina
  def eliminar
    @proyecto = Proyecto.find(params[:id])
    @proyecto.destroy
    if params[:convenio_id] 
      convenio = Proyecto.find_by_id params[:convenio_id]
      @proyectos = convenio.pacs
      render :update do |page|
        page.replace_html params[:update], :partial => "listado_proyectos_convenio"
        page.visual_effect :highlight, params[:update] , :duration => 6
        page.replace_html 'MB_content', :inline => '<%= mensaje_error(@proyecto, :eliminar => true) %><br>'
        page.call("Modalbox.resizeToContent")
      end
    else
      flash[:error_eliminar] = @proyecto
      redirect_to :action => 'listado'
    end
  end
  
  # en proyectos: lista los proyectos asociados al usuario declarado en la sessión
  def listado_usuario
    filtrado_condiciones
    @proyectos = @paginado = session[:usuario_identificado].proyecto.reload.paginate(:all, :conditions => @condiciones, :order => (session[:proyecto_orden] + " " + session[:proyecto_asc_desc] ) , :include => [:pais, :agente], :page => params[:page], :per_page => (session[:por_pagina] or 20) )
  end

  # en convenios: lista los proyectos asociados
  def listado_proyectos_convenio
    convenio = Proyecto.find_by_id params[:convenio_id]
    @proyectos = convenio.pacs
    render :update do |page|
      page.replace_html params[:update], :partial => "listado_proyectos_convenio"
    end

  end

end
