#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la visibilizacion y calculo de remanentes para Convenios
#--

class RemanenteController < ApplicationController

  helper :resumen_proyecto

	# en proyectos: presenta el calculo de remanentes segun la opcion de moneda elegida 
  def index 
    @listado_moneda = [[@proyecto.moneda_principal.nombre.pluralize,"todas"],["Todas las Monedas","locales"]]
    @listado_tipo = [[_("Tesoreria"),"tesoreria"],[_("Ejecución"), "ejecucion"]]
    @listado_tipo = [[_("Tesoreria"),"tesoreria"]]
    @listado_vista = [["Por Partidas", "partida"], ["Por Actividades", "actividad"]]
    @resumen = []

    # Sólo si hay un PAC anterior
    unless @proyecto.pac_anterior.nil? 
      params[:moneda] ||= "todas"
      params[:partida_actividad] ||= "partida"
      params[:tipo] ||= "tesoreria"

      # Remanentes por tesoreria
      if params[:tipo] == "tesoreria"
        (cab_col, cab_fila, filas) = @proyecto.pac_anterior.tesoreria(@proyecto.agente, nil, params[:moneda] == "todas")

        cabecera = [ [_("Remanentes de") + " " + @proyecto.pac_anterior.nombre,"1"] ]
        cab_col.each do |cab|
          p = Pais.find_by_id(cab[:pais_id])
          m = Moneda.find_by_id(cab[:moneda_id])
          cabecera.push( [ m.nombre + ( p ? " " + p.nombre : ""), "1_2_td"] )
        end

        lineas = []
        lineas.push( :cabecera => cabecera )
        for i in (0..3)
          estilo = [ ['',"1"] ]
          estilo += filas[i-4].collect{ ['','1_2_td' + (i==1 ? '_g' : '')] }
          lineas.push( :estilo => estilo, :contenido => [ cab_fila[i-4] ] + filas[i-4] )
        end
        nombre = _("Remanente de Tesorería") + " " + @proyecto.pac_anterior.nombre
        @resumen.push( :listado => {:nombre => nombre, :titulo => nombre, :lineas => lineas})

      # Remanentes por partida o actividad
      else
        estado_gasto
      end

      # Aplica los remanentes si asi se ha pedido
      if params[:commit] == _("Aplicar Remanentes")
        @proyecto.aplica_remanentes if @proyecto.pac_anterior
        flash[:mensaje] = _("Se han generado las transferencias correspondientes a los remanentes de") + " '" + @proyecto.pac_anterior.nombre + "'"
      end

      respond_to do |format|
        format.html do 
          render :action => "index", :layout => (params[:sin_layout] ? false : true)
        end unless params[:fichero] == '1' && params[:commit] == "Vista Previa"
        format.xls do
          @nombre = _("Cálculo Remanentes")
          render 'comunes_xls/resumen', :layout => false
        end if params[:fichero] == "1" && params[:commit] == "Vista Previa"
      end
    else
      render :action => "index", :layout => (params[:sin_layout] ? false : true)
    end
  end

	# en proyectos: obtiene el calculo de remanentes
  def estado_gasto
    # Inicializamos los datos necesarios
    fecha_de_inicio = @proyecto.pac_anterior.fecha_de_inicio
    fecha_de_fin = @proyecto.pac_anterior.fecha_de_fin
    fecha_de_fin = (fecha_de_fin >> 1) - 1 # Para que incluya el mes en elegido
    proyecto = @proyecto.pac_anterior

    # Las filas son partidas o actividades, segun se haya elegido
    filas = Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}} if params[:partida_actividad] == "partida"
    filas = proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}} if params[:partida_actividad] == "actividad"

    # Selecciona el tipo de calculo de remanentes (convertidas a moneda principal o por cada moneda)
    monedas = ["todas"] if params[:moneda] == "todas"
    monedas = @proyecto.moneda.collect{|m| m.id} unless params[:moneda] == "todas"
  
    # Construye una tabla por cada moneda 
    monedas.each do |moneda|
      datos = []
      columnas = []

      # Va recorriendo todos los implementadores para meterlos en cada columna
      columna = 1
      proyecto.implementador.each do |agente|
        # Si queremos ver remanentes por partidas
        if params[:partida_actividad] == "partida"
          presupuesto = VPresupuestoDetallado.partida_de_implementador( proyecto.id, moneda, agente.id, fecha_de_inicio, fecha_de_fin, "0")
          gasto = VGasto.partida_de_implementador( proyecto.id, moneda, agente.id, fecha_de_inicio, fecha_de_fin, "0")
        # Si queremos ver remanentes por actividades
        else
          presupuesto = VPresupuestoDetallado.actividad_de_implementador( proyecto.id, moneda, agente.id, fecha_de_inicio, fecha_de_fin , "0")
          gasto = VGasto.actividad_de_implementador( proyecto.id, moneda, agente.id, fecha_de_inicio, fecha_de_fin, "0")
        end
        # Mete en cada fila la diferencia entre presupuesto y gasto
        for fila in filas do
          p = presupuesto ? presupuesto.detect {|v| v["fila_id"] == fila["id"].to_s} : nil
          g = gasto ? gasto.detect {|v| v["fila_id"] == fila["id"].to_s} : nil
          datos.push( {"columna_id" => columna.to_s, "fila_id" => fila["id"].to_s, "importe" => (p ? p["importe"] : 0.0) - (g ? g["importe"] : 0.0) } )
        end
        columnas.push({"id" => columna.to_s, "nombre" => agente.nombre})
        columna += 1 
      end

      # Incluye la tabla de la moneda
      titulo =[	_("Remanentes de") + " " + proyecto.nombre,
		(moneda == "todas" ? _("Cálculo en") + ": " + proyecto.moneda_principal.nombre.pluralize : _("Remanentes de") + " " + Moneda.find_by_id(moneda.to_i).nombre) +
		" " + _("por") + " " + params[:partida_actividad].capitalize,
		_("Fechas") + ": " + fecha_de_inicio.to_s + '/' +  fecha_de_fin.to_s  ]

      @resumen.push(:tabla => {:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => {  :columna_resta => false, :columna_suma => true, :fila_suma => true, :columna_desviacion => false, :clases => ["","3_2","1_2_td"] } })
    end
  end

end
